
#include <sycl/sycl.hpp>
#include <iostream>

int main( int argc, char * argv[] ) {

  // If no argument, list the device ranks and exit

  if (argc==1) {
    std::size_t rank = 0 ;
    sycl::queue q { [&]( sycl::device const & device ) {
        std::string title { "Device " } ;
        title += std::to_string(rank++) ;
        auto const & p = device.get_platform() ;
        std::string name = device.get_info<sycl::info::device::name>();
        std::cout << title << ": " << name << "\n";
        return 0 ;
    }};
    return 0 ;
  }

  // Else, select the requested device

  std::size_t rank = 0, required_rank = std::stoul(argv[1]) ;
  sycl::queue q { [&]( sycl::device const & d ) {
      return (rank++==required_rank) ;
  }};
  std::cout
    << "Running: "
    << q.get_device().get_info<sycl::info::device::name>()
    << "\n";

  // The test

  sycl::buffer<sycl::opencl::cl_int, 1> buffer(4);
  sycl::range<1> nb_work_items{buffer.size()};
  q.submit([&](sycl::handler &cgh) {
    auto accessor = buffer.get_access<sycl::access::mode::write>(cgh);
    cgh.parallel_for<class FillBuffer>(
        nb_work_items, [=](sycl::id<1> wi_id) {
          accessor[wi_id] = (sycl::opencl::cl_int)wi_id.get(0);
        });
  });
  const auto hacc = buffer.get_host_access();
  bool mismatch = false;
  for (size_t i = 0; i < buffer.size(); ++i) {
    if (hacc[i] != i) {
      std::cout << "The result is incorrect for element: " << i
                << " , expected: " << i << " , got: " << hacc[i]
                << std::endl;
      mismatch = true;
    }
  }
  if (!mismatch) {
    std::cout << "The results are correct!" << std::endl;
  }
  return mismatch;
}

