
# Setup for Gray-Scott with SYCL

With SYCL, you benefit from easier coding of portable parallel code... and typically suffer from harder setup. This document try to help.

To be fully accompanied in this GrayScoot course, you should better have **Intel oneAPI** installed on your computer, or have access to a computing center where it is.

Another famous implementation of SYCL is [**AdaptativeCpp**](https://github.com/AdaptiveCpp/), but we have not tested it enough. You can use it if you prefer and know how to, but you are on your own.

Be aware that the basic Intel oneAPI installation will only see the Intel hardware, unless you additionally installed some specific plugins, such a the [codeplay ones](https://codeplay.com/solutions/oneapi/). In the following, we generally only address the NVidia/CUDA plugin. If you want to work with an AMD card, you are on your own.

Also, most of our instructions and utility scripts assume you are running a bash shell.

## [Online Code Reckons compiler](https://codereckons.com/tools)

If you have an account on Code Reckons, you can simply copy/paste your code in the [online compiler](https://codereckons.com/tools). Just make sure to **select the ICPX compiler**.

Tip: so to see the output of your program, in your C++ source window, ask for "Add new... Execution Only".

Try to compile and run the content of `CheckOneApiCuda/src/main.cpp`. If you run it without any arguments, you will get a list of available devices with their *ranks*. If you provide a command-line rank, the program will try to run a test on the device with this rank. Because the default compilation options on Code Reckons only generate the code for CUDA, you will see that the program will not be able to run on the CPU.

## Running on Intel DevCloud

Provided you have asked for an account ahead of time, on the [Intel Tiber Developer Cloud for oneAPI](https://devcloud.intel.com/oneapi/home/), and made the Intel recommended local ssh installation, that is how you may run the demo `SquareBuffers` on an intel integrated GPU:

```
ssh devcloud
git clone https://gitlab.in2p3.fr/chamont/grayscottchallenge.git
cd GrayScottChallenge
qsub -l nodes=1:gpu:ppn=2 -d . cmake-oneapi.sh
cd SquareBuffers
qsub -l nodes=1:gpu:ppn=2 -d . make.sh
qsub -l nodes=1:gpu:ppn=2 -d . run.sh
qstat
qdel ...
```

Or, if you prefer to run on cpu:

```
...
qsub -l nodes=1:xeon:ppn=2 -d . cmake-oneapi.sh
...
qsub -l nodes=1:xeon:ppn=2 -d . make.sh
qsub -l nodes=1:xeon:ppn=2 -d . run.sh
...
```

## Using MUST conputing center


## Running Intel oneAPI with docker

If you want to run within a container, a set of predefined images are usable:
- the [default oneAPI image](intel/oneapi-basekit) (4.26 GiB) which see your Intel CPU and iGPU.
- a ["smaller" oneAPI image](gitlab-registry.in2p3.fr/codeursintensifs/grayscott/grayscottsyclsetup/jfalcou:2024.1) (2.29 GiB) which only see your Intel CPU.
- a ["bigger" oneAPI image](gitlab-registry.in2p3.fr/codeursintensifs/grayscott/grayscottsyclsetup/dchamont:2024.1) (5.29 GiB) which add the NVidia codeplay plugin, and therefore also see your NVidia card.

We recommend you to start your container with the following command, which sets some useful options for Intel GPU (`--device=/dev/dri`) and|or NVidia card (`--gpus all`), and mount the current directory as `/work` in the container:

```sh
IMG=gitlab-registry.in2p3.fr/codeursintensifs/grayscott/grayscottsyclsetup/dchamont:2024.1
docker pull ${IMG}
docker run --gpus all --device=/dev/dri --network host -it --rm -v ${PWD}:/work -w /work ${IMG}
```

Check that the installation is OK with `sycl-ls`, and `nvidia-smi` if you have a CUDA card.

## Intel oneAPI Base Toolkit bare-metal installation

### Intel oneAPI basekit

Follow the [recipe provided by Intel](https://www.intel.com/content/www/us/en/developer/tools/oneapi/toolkits.html#base-kit).

For example:
- download the shell script for a Linux  "Online installer", version 2024.1.0.
- run it with the recommended silent options,
- add `. /opt/intel/oneapi/setvars.sh --include-intel-llvm` in your `.bashrc`.

Check that the installation seems OK with `sycl-ls`.

Note: when trying the recipe above, `sycl-ls` does not show the Intel iGPU. I wonder why.

### Cuda

...

Check with `nvidia-smi` that your NVidia card is seen.

### Codeplay plugin for NVidia

Get the installer from `https://developer.codeplay.com/products/oneapi/nvidia/download`,
and run it with `sh ...`.

Then check with `sycl-ls` that your NVidia card is seen.









## Check SYCL installation and init your project

Before anything, `source env.bash` within the top directory.
Then, depending on the available SYCL implementation:
- oneAPI: `cmake-oneapi.bash`
- Open SYCL: `cmake-opensycl.bash`

Then move to `SquareBuffers`, inspect all the files so to get acquainted with the
build chain, and check that your installation is working:

```
./make.bash
./run.bash
```

Duplicate the directory `SquareBuffers` as `MyProject`,
an check again you can build and run this new project. This is
where you will add your own code from now on.

## See how to select different devices

Move to `DeviceList`, build and run, so to see all the devices which are
available on your machine, and some of their properties. Especially, check
the rating which is returned by the different predefined selectors.

Note that the oneAPI implementation provides a command called `sycl-ls`,
which displays a short list of available devices. Try it.

Then move to `DeviceSelect`, which demonstrates different ways to select
a device : the predefined ones, and a home-made one (lambda function)
which select the device whose rank is provided on the command line.
The specific `run.bash` of this direcotry require rank number as
argument. so to demonstrate how a device can be selected this way.
The program also lists all the devces which are insvestigated while
constructing a queue. It should be the same, or close to what was
shown by `DeviceList`... but you may discover that it is not always
the case (at least with the Open SYCL implementaiton we just tried !).

Improve `MyProject` with the selector of your choice.

